"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PopoverView = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _propTypes = _interopRequireDefault(require("prop-types"));

var Helpers = _interopRequireWildcard(require("./functions/helpers"));

var _UIManager$componentN;

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const componentName = "RCTPopoverView";
const NativeCommands = (_UIManager$componentN = _reactNative.UIManager[componentName]) === null || _UIManager$componentN === void 0 ? void 0 : _UIManager$componentN.Commands;
const NativeComponent = (0, _reactNative.requireNativeComponent)(componentName);
const moduleName = "PopoverModule";
const PopoverModule = _reactNative.NativeModules[moduleName];
const COMP_COMMAND_KEYS = {
  'setVisibility': 'setVisibility'
};
const MODULE_COMMAND_KEYS = {
  'setVisibility': 'setVisibility',
  'getVisibility': 'getVisibility'
};

class PopoverView extends _react.default.PureComponent {
  constructor(props) {
    super(props);

    _defineProperty(this, "setVisibility", async visibility => {
      const {
        lazyPopover
      } = this.props;

      try {
        if (visibility) {
          await Promise.all([Helpers.setStateAsync(this, {
            mountPopover: true
          }), // temp bugfix: wait for popover to mount
          lazyPopover && Helpers.timeout(50)]);
        }

        ;
        await PopoverModule[MODULE_COMMAND_KEYS.setVisibility]((0, _reactNative.findNodeHandle)(this.nativeRef), visibility);
      } catch (error) {
        if (__DEV__) {
          console.warn("PopoverView, setVisibility", error);
        }

        ;
        throw error;
      }

      ;
    });

    _defineProperty(this, "toggleVisibility", async () => {
      const visibility = await this.getVisibility();
      await this.setVisibility(!visibility);
    });

    _defineProperty(this, "getVisibility", async () => {
      return await PopoverModule[MODULE_COMMAND_KEYS.getVisibility]((0, _reactNative.findNodeHandle)(this.nativeRef));
    });

    _defineProperty(this, "_handleOnPopoverDidHide", () => {
      var _this$props$onPopover, _this$props;

      (_this$props$onPopover = (_this$props = this.props).onPopoverDidHide) === null || _this$props$onPopover === void 0 ? void 0 : _this$props$onPopover.call(_this$props);
      Helpers.setStateAsync(this, {
        mountPopover: false
      });
    });

    this.state = {
      /** controls whether the popover content is mounted */
      mountPopover: false
    };
  }

  componentWillUnmount() {
    this.setVisibility(false);
  }

  //#endregion
  render() {
    var _props$renderPopoverC;

    const props = this.props;
    const {
      mountPopover
    } = this.state;
    const nativeProps = {
      // Values -----------------------------------------------------------
      popoverBackgroundColor: (0, _reactNative.processColor)(props.popoverBackgroundColor),
      permittedArrowDirections: props.permittedArrowDirections,
      // handle `popoverSize` prop...
      ...(Helpers.isObject(props.popoverSize) // `props.popoverSize` is width/height obj
      ? {
        popoverSize: 'CUSTOM',
        customPopoverSize: props.popoverSize
      } // `props.popoverSize` is string i.e. PopoverSize enum value
      : {
        popoverSize: props.popoverSize
      }),
      // Flags ----------------------------------------
      popoverShouldDismiss: props.popoverShouldDismiss,
      popoverCanOverlapSourceViewRect: props.popoverCanOverlapSourceViewRect,
      // Events --------------------------------------------
      onPopoverDidHide: this._handleOnPopoverDidHide,
      onPopoverWillShow: props.onPopoverWillShow,
      onPopoverWillHide: props.onPopoverWillHide,
      onPopoverDidShow: props.onPopoverDidShow,
      onPopoverDidHideViaTap: props.onPopoverDidHideViaTap,
      onPopoverWillHideViaTap: props.onPopoverWillHideViaTap,
      onPopoverDidAttemptToDismiss: props.onPopoverDidAttemptToDismiss
    };
    return /*#__PURE__*/_react.default.createElement(NativeComponent, _extends({
      ref: r => this.nativeRef = r
    }, nativeProps), /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: styles.popoverContainer
    }, (mountPopover || !props.lazyPopover) && ((_props$renderPopoverC = props.renderPopoverContent) === null || _props$renderPopoverC === void 0 ? void 0 : _props$renderPopoverC.call(props))), this.props.children);
  }

}

exports.PopoverView = PopoverView;

_defineProperty(PopoverView, "proptypes", {
  popoverSize: _propTypes.default.oneOf([_propTypes.default.object, _propTypes.default.string]),
  popoverBackgroundColor: _propTypes.default.string,
  permittedArrowDirections: _propTypes.default.arrayOf(_propTypes.default.string),
  // flags -------------------
  lazyPopover: _propTypes.default.bool,
  popoverShouldDismiss: _propTypes.default.bool,
  popoverCanOverlapSourceViewRect: _propTypes.default.bool,
  // functions/events --------------------
  onPopoverDidShow: _propTypes.default.func,
  onPopoverDidHide: _propTypes.default.func,
  onPopoverWillShow: _propTypes.default.func,
  onPopoverWillHide: _propTypes.default.func,
  renderPopoverContent: _propTypes.default.func,
  onPopoverDidHideViaTap: _propTypes.default.func,
  onPopoverWillHideViaTap: _propTypes.default.func,
  onPopoverDidAttemptToDismiss: _propTypes.default.func
});

_defineProperty(PopoverView, "defaultProps", {
  lazyPopover: true,
  popoverShouldDismiss: true
});

;

const styles = _reactNative.StyleSheet.create({
  popoverContainer: {
    position: 'absolute',
    color: 'transparent'
  }
});
//# sourceMappingURL=PopoverView.ios.js.map