var _UIManager$componentN;

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { NativeModules, UIManager, StyleSheet, requireNativeComponent, findNodeHandle, processColor, View } from 'react-native';
import Proptypes from 'prop-types';
import * as Helpers from './functions/helpers';
const componentName = "RCTPopoverView";
const NativeCommands = (_UIManager$componentN = UIManager[componentName]) === null || _UIManager$componentN === void 0 ? void 0 : _UIManager$componentN.Commands;
const NativeComponent = requireNativeComponent(componentName);
const moduleName = "PopoverModule";
const PopoverModule = NativeModules[moduleName];
const COMP_COMMAND_KEYS = {
  'setVisibility': 'setVisibility'
};
const MODULE_COMMAND_KEYS = {
  'setVisibility': 'setVisibility',
  'getVisibility': 'getVisibility'
};
export class PopoverView extends React.PureComponent {
  constructor(props) {
    super(props);

    _defineProperty(this, "setVisibility", async visibility => {
      const {
        lazyPopover
      } = this.props;

      try {
        if (visibility) {
          await Promise.all([Helpers.setStateAsync(this, {
            mountPopover: true
          }), // temp bugfix: wait for popover to mount
          lazyPopover && Helpers.timeout(50)]);
        }

        ;
        await PopoverModule[MODULE_COMMAND_KEYS.setVisibility](findNodeHandle(this.nativeRef), visibility);
      } catch (error) {
        if (__DEV__) {
          console.warn("PopoverView, setVisibility", error);
        }

        ;
        throw error;
      }

      ;
    });

    _defineProperty(this, "toggleVisibility", async () => {
      const visibility = await this.getVisibility();
      await this.setVisibility(!visibility);
    });

    _defineProperty(this, "getVisibility", async () => {
      return await PopoverModule[MODULE_COMMAND_KEYS.getVisibility](findNodeHandle(this.nativeRef));
    });

    _defineProperty(this, "_handleOnPopoverDidHide", () => {
      var _this$props$onPopover, _this$props;

      (_this$props$onPopover = (_this$props = this.props).onPopoverDidHide) === null || _this$props$onPopover === void 0 ? void 0 : _this$props$onPopover.call(_this$props);
      Helpers.setStateAsync(this, {
        mountPopover: false
      });
    });

    this.state = {
      /** controls whether the popover content is mounted */
      mountPopover: false
    };
  }

  componentWillUnmount() {
    this.setVisibility(false);
  }

  //#endregion
  render() {
    var _props$renderPopoverC;

    const props = this.props;
    const {
      mountPopover
    } = this.state;
    const nativeProps = {
      // Values -----------------------------------------------------------
      popoverBackgroundColor: processColor(props.popoverBackgroundColor),
      permittedArrowDirections: props.permittedArrowDirections,
      // handle `popoverSize` prop...
      ...(Helpers.isObject(props.popoverSize) // `props.popoverSize` is width/height obj
      ? {
        popoverSize: 'CUSTOM',
        customPopoverSize: props.popoverSize
      } // `props.popoverSize` is string i.e. PopoverSize enum value
      : {
        popoverSize: props.popoverSize
      }),
      // Flags ----------------------------------------
      popoverShouldDismiss: props.popoverShouldDismiss,
      popoverCanOverlapSourceViewRect: props.popoverCanOverlapSourceViewRect,
      // Events --------------------------------------------
      onPopoverDidHide: this._handleOnPopoverDidHide,
      onPopoverWillShow: props.onPopoverWillShow,
      onPopoverWillHide: props.onPopoverWillHide,
      onPopoverDidShow: props.onPopoverDidShow,
      onPopoverDidHideViaTap: props.onPopoverDidHideViaTap,
      onPopoverWillHideViaTap: props.onPopoverWillHideViaTap,
      onPopoverDidAttemptToDismiss: props.onPopoverDidAttemptToDismiss
    };
    return /*#__PURE__*/React.createElement(NativeComponent, _extends({
      ref: r => this.nativeRef = r
    }, nativeProps), /*#__PURE__*/React.createElement(View, {
      style: styles.popoverContainer
    }, (mountPopover || !props.lazyPopover) && ((_props$renderPopoverC = props.renderPopoverContent) === null || _props$renderPopoverC === void 0 ? void 0 : _props$renderPopoverC.call(props))), this.props.children);
  }

}

_defineProperty(PopoverView, "proptypes", {
  popoverSize: Proptypes.oneOf([Proptypes.object, Proptypes.string]),
  popoverBackgroundColor: Proptypes.string,
  permittedArrowDirections: Proptypes.arrayOf(Proptypes.string),
  // flags -------------------
  lazyPopover: Proptypes.bool,
  popoverShouldDismiss: Proptypes.bool,
  popoverCanOverlapSourceViewRect: Proptypes.bool,
  // functions/events --------------------
  onPopoverDidShow: Proptypes.func,
  onPopoverDidHide: Proptypes.func,
  onPopoverWillShow: Proptypes.func,
  onPopoverWillHide: Proptypes.func,
  renderPopoverContent: Proptypes.func,
  onPopoverDidHideViaTap: Proptypes.func,
  onPopoverWillHideViaTap: Proptypes.func,
  onPopoverDidAttemptToDismiss: Proptypes.func
});

_defineProperty(PopoverView, "defaultProps", {
  lazyPopover: true,
  popoverShouldDismiss: true
});

;
const styles = StyleSheet.create({
  popoverContainer: {
    position: 'absolute',
    color: 'transparent'
  }
});
//# sourceMappingURL=PopoverView.ios.js.map